#include "particledata.h"
#include "sph.h"
#include "read_params.h"
#include "split.h"
#include "convertTo.h"

#include <iostream>
#include <fstream>
#include <stdexcept> 
#include <map>
#include <string>
#include <vector>

using std::cout; 
using std::endl; 
using std::cerr; 
using std::ifstream; 
using std::domain_error; 
using std::map; 
using std::pair; 
using std::string; 
using std::getline; 
using std::vector; 

enum var_type {
  INT, 
  DOUBLE, 
  STRING
}; 

struct AdressAndType {
  void *var_adr; 
  enum var_type type; 
}; 

// Read gloval variables defined in particledata.h sph.h from 
// a parameter file. 
ifstream& read_params(ifstream& in)
{
  map<string, AdressAndType> global_variables; 
  AdressAndType aat; 

  aat.var_adr = &OutputFileBase; 
  aat.type = STRING; 
  global_variables["OutputFileBase"] = aat;  

  Gamma = 0; 
  aat.var_adr = &Gamma; 
  aat.type = DOUBLE; 
  global_variables["Gamma"] = aat;  

  ArtBulkViscConst = 0; 
  aat.var_adr = &ArtBulkViscConst; 
  aat.type = DOUBLE; 
  global_variables["ArtBulkViscConst"] = aat; 

  CourantFac = 0; 
  aat.var_adr = &CourantFac; 
  aat.type = DOUBLE; 
  global_variables["CourantFac"] = aat; 

  BegTime = 0; 
  aat.var_adr = &BegTime; 
  aat.type = DOUBLE; 
  global_variables["BegTime"] = aat; 
  
  MaxTime = 0; 
  aat.var_adr = &MaxTime; 
  aat.type = DOUBLE; 
  global_variables["MaxTime"] = aat; 

  DtOutput = 0; 
  aat.var_adr = &DtOutput; 
  aat.type = DOUBLE; 
  global_variables["DtOutput"] = aat; 

  Nngb = 0; 
  aat.var_adr = &Nngb; 
  aat.type = DOUBLE; 
  global_variables["Nngb"] = aat; 

#ifdef REDUCEVISC
  MinAlpha = 0; 
  aat.var_adr = &MinAlpha; 
  aat.type = DOUBLE; 
  global_variables["MinAlpha"] = aat; 

  /*
  ViscParam = 0;  
  aat.var_adr = &ViscParam; 
  aat.type = DOUBLE; 
  global_variables["ViscParam"] = aat; 
  */

  ViscDecayLength = 0; 
  aat.var_adr = &ViscDecayLength; 
  aat.type = DOUBLE; 
  global_variables["ViscDecayLength"] = aat; 

#endif

#ifdef AC
  ArtBulkCondConst = 0; 
  aat.var_adr = &ArtBulkCondConst; 
  aat.type = DOUBLE; 
  global_variables["ArtBulkCondConst"] = aat; 
#endif

#if defined(ONE_D_SHOCKTUBE) || defined(TWO_D_KH) || defined(THREE_D_SEDOV) 
  Ngrid = 0; 
  aat.var_adr = &Ngrid; 
  aat.type = INT; 
  global_variables["Ngrid"] = aat; 
#endif 

  string line; 
  int count = 0;  
  while (getline(in, line)) { 
    cout << "line = " << line << endl; 
    vector<string> v; 
    split(line, back_inserter(v));  
    cout << "v.size() = " << v.size() << endl; 
    if (v.size() < 2)
      continue; 
    else if (v[0][0] == '%')
      continue; 

    map<string, AdressAndType>::iterator gvar_iter = global_variables.find(v[0]); 

    if (gvar_iter == global_variables.end()) {
      cerr << "Variable: " << v[0] << " "; 
      throw domain_error("does not exist!!"); 
    } else {
      switch (gvar_iter->second.type) {
        case INT:
          {
            int *ivar = static_cast<int *>(gvar_iter->second.var_adr); 
            if (*ivar != 0) {
              cerr << "ivalue = " << *ivar << endl; 
              cerr << "Variable: " << v[0] << " "; 
              throw domain_error("is douplicated!"); 
            }
            *ivar = convertTo(v[1], *ivar); 
          } 
          ++count; 
          break; 
        case DOUBLE:
          {
            double *dvar = static_cast<double *>(gvar_iter->second.var_adr); 
            if (*dvar != 0) {
              cerr << "Variable: " << v[0] << " "; 
              throw domain_error("is douplicated!"); 
            }
            *dvar = convertTo(v[1], *dvar); 
          }
          ++count; 
          break; 
        case STRING:
          { 
            string *svar = static_cast<string *>(gvar_iter->second.var_adr); 
            if (svar->size() != 0) {
                cerr << "Variable: " << v[0] << " "; 
              throw domain_error("is douplicated!"); 
            }
            *svar = v[1]; 
          }
          ++count; 
          break; 
      } 
    }
  }    
  
  if (count < global_variables.size()) { 
    cout << "count = " << count << " global_variables.size() = " << global_variables.size() << endl; 
    throw domain_error("Your parameter is likely to miss some variables!!"); 
  }

  return in; 
}

void set_global(void)
{
  Gamma_1 = Gamma - 1;
  MinNngb = Nngb * pow(0.99, static_cast<double>(DIM)); 
  MaxNngb = Nngb * pow(1.01, static_cast<double>(DIM));  
#ifdef REDUCEVISC
  ViscParam = (1.e0/ViscDecayLength) * sqrt(Gamma_1 / (2 * Gamma)); 
  cout << "ViscParam = " << ViscParam << endl; 
#endif
}
