#ifndef SPH_H
#define SPH_H
//-------------------------------------
#include "tree.h"
#include "particledata.h"
#include <vector>


#if (DIM == 1) 
const double KernelCoeff = 4./3.; 
const double NormCoeff = 2; 
#elif (DIM == 2) 
const double KernelCoeff = 40./(7 * M_PI); 
const double NormCoeff = M_PI; 
#elif (DIM == 3)
const double KernelCoeff = 8./M_PI; 
const double NormCoeff = (4 * M_PI / 3.); 
#endif

const double Epsilon = 1.e-4; // just a small number 

void CalcDensity(const std::vector<NodeInfo>& tree, const std::vector<ParticleList>& plist, std::vector<ParticleData>& sph); 
void CalcHydroForce(const std::vector<NodeInfo>& tree, const std::vector<ParticleList>& plist, std::vector<ParticleData>& sph); 
#ifdef USE_SMOOTHED_P
void CalcSmoothedPresure(const std::vector<NodeInfo>& tree, const std::vector<ParticleList>& plist, std::vector<ParticleData>& sph); 
#endif

inline double H3(const double& h)
{
#if (DIM == 1) 
  return h; 
#elif (DIM == 2)
  return h*h; 
#elif (DIM == 3)
  return h*h*h; 
#endif
}

//-------------------------------------
#endif
